from datetime import datetime, date, timedelta

from .timetable import MultipleIterator, TT_STORE_BEFORE, TT_STORE_AFTER
from .backends import edt_ade, clubs, recurring, formation

"""
Timetable frontend, with some utility function and classes.
"""

BACKENDS = [edt_ade, clubs, recurring, formation]


class InvalidTimetablesException(Exception):
    """
    Exception raised when some timetables are invalid.
    """


def get_timetables_for(user):
    """
    Get the timetables for a user, as a list.
    The other utility functions may be easier to use.
    """

    result = []
    for backend in BACKENDS:
        backend.get_timetables_for(user, result)

    errors = []
    for timetable in result:
        if not timetable.is_valid:
            errors.append(f"{timetable.title} ({timetable.reason})")

    if errors:
        errors = ", ".join(errors)
        raise InvalidTimetablesException(f"Invalid timetables: {errors}")

    return result


def get_next_event_for_user(user, offset=0):
    """
    Gets the next event of interest for a user.
    """

    timetables = get_timetables_for(user)

    return get_next_event(timetables, offset)


def get_next_event_for_edt_ade_group(gid, school, offset=0):
    """
    Gets the next event of interest for a group and school.
    """

    timetable = edt_ade.get_timetables_for_gid(gid, school)
    if timetable is None:
        raise KeyError(gid)

    return get_next_event([timetable], offset)


def get_next_event(timetables, offset=0):
    """
    Gets the next event of interest for a user.
    """

    for event, _ in MultipleIterator(timetables, datetime.now()):
        if offset == 0:
            return event

        offset -= 1

    return None


def get_day_events_for(user, day_offset=0, ignore_conflicting=False):
    """
    Gets all events for a specific day.
    """

    timetables = get_timetables_for(user)

    if not timetables:
        return []

    selected_date = date.today() + timedelta(days=day_offset)
    date_parts = selected_date.timetuple()[0:3]
    iterator = MultipleIterator(timetables, datetime(*date_parts))

    def generator_func():
        prev_event = None

        for event, _ in iterator:
            if event.start.date() != selected_date:
                break

            if ignore_conflicting and prev_event and event.start < prev_event.end:
                continue

            prev_event = event
            yield event

    return generator_func()


def get_week_info(user, week_offset):
    """
    Given a week offset, returns a tuple with the following items:
    - the relevant event lists for the week offset
    - True if the week offset is still valid if decremented
    - True if the week offset is still valid if incremented
    """

    if -TT_STORE_BEFORE <= week_offset <= TT_STORE_AFTER:
        timetables = get_timetables_for(user)
        offset = week_offset + TT_STORE_BEFORE
        event_lists = [timetable.get_week(offset) for timetable in timetables]

        return (
            event_lists,
            week_offset > -TT_STORE_BEFORE,
            week_offset < TT_STORE_AFTER,
        )

    return None, False, False
